=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-08-15 13:20:18 +0000
+++ bzrlib/tests/__init__.py	2006-08-17 11:10:57 +0000
@@ -216,6 +216,7 @@
         if isinstance(err[1], TestSkipped):
             return self.addSkipped(test, err)    
         unittest.TestResult.addError(self, test, err)
+        test.setKeepLogfile()
         self.extractBenchmarkTime(test)
         if self.showAll:
             self.stream.writeln("ERROR %s" % self._testTimeString())
@@ -232,6 +233,7 @@
 
     def addFailure(self, test, err):
         unittest.TestResult.addFailure(self, test, err)
+        test.setKeepLogfile()
         self.extractBenchmarkTime(test)
         if self.showAll:
             self.stream.writeln(" FAIL %s" % self._testTimeString())
@@ -451,6 +453,7 @@
 
     _log_file_name = None
     _log_contents = ''
+    _keep_log_file = False
     # record lsprof data when performing benchmark calls.
     _gather_lsprof_in_benchmarks = False
 
@@ -567,16 +570,20 @@
     def _finishLogFile(self):
         """Finished with the log file.
 
-        Read contents into memory, close, and delete.
+        Close the file and delete it, unless setKeepLogfile was called.
         """
         if self._log_file is None:
             return
         bzrlib.trace.disable_test_log(self._log_nonce)
-        self._log_file.seek(0)
-        self._log_contents = self._log_file.read()
         self._log_file.close()
-        os.remove(self._log_file_name)
-        self._log_file = self._log_file_name = None
+        self._log_file = None
+        if not self._keep_log_file:
+            os.remove(self._log_file_name)
+            self._log_file_name = None
+
+    def setKeepLogfile(self):
+        """Make the logfile not be deleted when _finishLogFile is called."""
+        self._keep_log_file = True
 
     def addCleanup(self, callable):
         """Arrange to run a callable when this case is torn down.
@@ -663,13 +670,25 @@
     def log(self, *args):
         mutter(*args)
 
-    def _get_log(self):
+    def _get_log(self, keep_log_file=False):
         """Return as a string the log for this test"""
-        if self._log_file_name:
-            return open(self._log_file_name).read()
-        else:
+        # flush the log file, to get all content
+        import bzrlib.trace
+        bzrlib.trace._trace_file.flush()
+        if self._log_contents:
             return self._log_contents
-        # TODO: Delete the log after it's been read in
+        if self._log_file_name is not None:
+            logfile = open(self._log_file_name)
+            try:
+                log_contents = logfile.read()
+            finally:
+                logfile.close()
+            if not keep_log_file:
+                self._log_contents = log_contents
+                os.remove(self._log_file_name)
+            return log_contents
+        else:
+            return "DELETED log file to reduce memory footprint"
 
     def capture(self, cmd, retcode=0):
         """Shortcut that splits cmd into words, runs, and returns stdout"""

=== modified file 'bzrlib/tests/blackbox/test_exceptions.py'
--- bzrlib/tests/blackbox/test_exceptions.py	2006-08-11 10:03:44 +0000
+++ bzrlib/tests/blackbox/test_exceptions.py	2006-08-17 11:10:57 +0000
@@ -49,7 +49,8 @@
             os.mkdir('foo')
             bzrdir.BzrDirFormat5().initialize('foo')
             out, err = self.run_bzr("status", "foo")
-            self.assertContainsRe(self._get_log(), "bzr upgrade")
+            self.assertContainsRe(self._get_log(keep_log_file=True),
+                                  "bzr upgrade")
         finally:
             repository._deprecation_warning_done = True
 

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2006-07-02 19:31:42 +0000
+++ bzrlib/tests/test_merge.py	2006-08-17 11:10:57 +0000
@@ -133,12 +133,12 @@
         log = StringIO()
         merge_inner(tree_b.branch, tree_a, tree_b.basis_tree(), 
                     this_tree=tree_b, ignore_zero=True)
-        log = self._get_log()
+        log = self._get_log(keep_log_file=True)
         self.failUnless('All changes applied successfully.\n' not in log)
         tree_b.revert([])
         merge_inner(tree_b.branch, tree_a, tree_b.basis_tree(), 
                     this_tree=tree_b, ignore_zero=False)
-        log = self._get_log()
+        log = self._get_log(keep_log_file=True)
         self.failUnless('All changes applied successfully.\n' in log)
 
     def test_merge_inner_conflicts(self):

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-07-24 11:26:40 +0000
+++ bzrlib/tests/test_selftest.py	2006-08-17 11:10:57 +0000
@@ -57,7 +57,8 @@
         """Test logs are captured when a test fails."""
         self.log('a test message')
         self._log_file.flush()
-        self.assertContainsRe(self._get_log(), 'a test message\n')
+        self.assertContainsRe(self._get_log(keep_log_file=True),
+                              'a test message\n')
 
 
 class TestTreeShape(TestCaseInTempDir):

=== modified file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	2006-06-20 03:30:14 +0000
+++ bzrlib/tests/test_trace.py	2006-08-17 11:10:57 +0000
@@ -80,8 +80,8 @@
     def test_trace_unicode(self):
         """Write Unicode to trace log"""
         self.log(u'the unicode character for benzene is \N{BENZENE RING}')
-        self.assertContainsRe('the unicode character',
-                self._get_log())
+        self.assertContainsRe(self._get_log(keep_log_file=True),
+                              "the unicode character for benzene is")
 
     def test_report_broken_pipe(self):
         try:
@@ -97,12 +97,7 @@
         # raise an exception
         mutter(u'Writing a greek mu (\xb5) works in a unicode string')
         mutter('But fails in an ascii string \xb5')
-        # TODO: jam 20051227 mutter() doesn't flush the log file, and
-        #       self._get_log() opens the file directly and reads it.
-        #       So we need to manually flush the log file
-        import bzrlib.trace
-        bzrlib.trace._trace_file.flush()
-        log = self._get_log()
+        log = self._get_log(keep_log_file=True)
         self.assertContainsRe(log, 'Writing a greek mu')
         self.assertContainsRe(log, 'UnicodeError')
         self.assertContainsRe(log, "'But fails in an ascii string")

